<?php

namespace Drupal\media_default_image\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\file\FileInterface;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Default Image' formatter.
 */
#[FieldFormatter(
  id: 'media_default_image_default_image',
  label:  new TranslatableMarkup("Default Image"),
  field_types: [
    'entity_reference',
  ]
)]
class DefaultImageFormatter extends ImageFormatterBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    private readonly AccountInterface $currentUser,
    private readonly EntityStorageInterface $imageStyleStorage,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly LoggerChannelFactoryInterface $logger,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('current_user'),
      $container->get('entity_type.manager')->getStorage('image_style'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'image_style' => '',
      'image_link' => '',
      'image_loading' => [
        'attribute' => 'lazy',
      ],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    $image_styles = image_style_options(FALSE);
    $description_link = Link::fromTextAndUrl(
      $this->t('Configure Image Styles'),
      Url::fromRoute('entity.image_style.collection')
    );
    $element['image_style'] = [
      '#title' => $this->t('Image style'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('image_style'),
      '#empty_option' => $this->t('None (original image)'),
      '#options' => $image_styles,
      '#description' => $description_link->toRenderable() + [
        '#access' => $this->currentUser->hasPermission('administer image styles'),
      ],
    ];

    $image_loading = $this->getSetting('image_loading');
    $element['image_loading'] = [
      '#type' => 'details',
      '#title' => $this->t('Image loading'),
      '#weight' => 10,
      '#description' => $this->t('Lazy render images with native image loading attribute (<em>loading="lazy"</em>). This improves performance by allowing browsers to lazily load images.'),
    ];
    $loading_attribute_options = [
      'lazy' => $this->t('Lazy (<em>loading="lazy"</em>)'),
      'eager' => $this->t('Eager (<em>loading="eager"</em>)'),
    ];
    $element['image_loading']['attribute'] = [
      '#title' => $this->t('Image loading attribute'),
      '#type' => 'radios',
      '#default_value' => $image_loading['attribute'],
      '#options' => $loading_attribute_options,
      '#description' => $this->t('Select the loading attribute for images. <a href=":link">Learn more about the loading attribute for images.</a>', [
        ':link' => 'https://html.spec.whatwg.org/multipage/urls-and-fetching.html#lazy-loading-attributes',
      ]),
    ];
    $element['image_loading']['attribute']['lazy']['#description'] = $this->t('Delays loading the image until that section of the page is visible in the browser. When in doubt, lazy loading is recommended.');
    $element['image_loading']['attribute']['eager']['#description'] = $this->t('Force browsers to download an image as soon as possible. This is the browser default for legacy reasons. Only use this option when the image is always expected to render.');

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $image_styles = image_style_options(FALSE);
    // Unset possible 'No defined styles' option.
    unset($image_styles['']);
    // Styles could be lost because of enabled/disabled modules that defines
    // their styles in code.
    $image_style_setting = $this->getSetting('image_style');
    if (isset($image_styles[$image_style_setting])) {
      $summary[] = $this->t('Image style: @style', ['@style' => $image_styles[$image_style_setting]]);
    }
    else {
      $summary[] = $this->t('Original image');
    }

    $image_loading = $this->getSetting('image_loading');
    $summary[] = $this->t('Image loading: @attribute', [
      '@attribute' => $image_loading['attribute'],
    ]);

    return array_merge($summary, parent::settingsSummary());
  }

  /**
   * {@inheritdoc}
   *
   * Override to remove \EntityReferenceItem::isDisplayed() that
   * not exist in this format.
   */
  protected function needsEntityLoad(EntityReferenceItem $item) {
    return !$item->hasNewEntity();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    if ($items instanceof EntityReferenceFieldItemListInterface) {
      $medias = $this->getEntitiesToView($items, $langcode);

      $entity = $items->getEntity();
      $imageStyleSetting = $this->getSetting('image_style');

      // Handle case with no media items.
      if (empty($medias)) {
        return $this->createDefaultImageElement($entity, $imageStyleSetting);
      }

      // Prepare for media elements.
      $baseCacheTags = [];
      if (!empty($imageStyleSetting)) {
        $imageStyle = $this->imageStyleStorage->load($imageStyleSetting);
        $baseCacheTags = $imageStyle->getCacheTags();
      }

      foreach ($medias as $delta => $file) {
        $cacheTags = Cache::mergeTags($baseCacheTags, $file->getCacheTags());
        $item = $file->get('field_media_image')->first();

        $itemAttributes = $item->_attributes;
        unset($item->_attributes);

        $itemAttributes['loading'] = $this->getSetting('image_loading')['attribute'];

        $elements[$delta] = [
          '#theme' => 'image_formatter',
          '#item' => $item,
          '#item_attributes' => $itemAttributes,
          '#image_style' => $imageStyleSetting,
          '#cache' => ['tags' => $cacheTags],
        ];
      }
    }
    return $elements;
  }

  /**
   * Creates a default image element with optional styling and linking.
   *
   * @param object $entity
   *   The entity for which the default image element is being created.
   * @param string|null $imageStyleSetting
   *   The image style setting to apply, or NULL if no styling is required.
   *
   * @return array
   *   A renderable array representing the default image element.
   */
  private function createDefaultImageElement($entity, $imageStyleSetting) {
    $elements = [];
    $delta = 0;

    try {
      // Obtenir le fichier d'image par défaut.
      $file = $this->getDefaultFile();
      $imageUri = $file->getFileUri();
      if (!empty($imageStyleSetting)) {
        $elements[$delta] = [
          '#theme' => 'image_style',
          '#style_name' => $imageStyleSetting,
          '#uri' => $imageUri,
          '#alt' => $entity->label(),
          '#title' => $entity->label(),
        ];
      }
      else {
        // Si aucun style n'est appliqué, utilise directement l'image.
        $elements[$delta] = [
          '#theme' => 'image',
          '#uri' => $imageUri,
          '#alt' => $entity->label(),
          '#title' => $entity->label(),
        ];
      }
    }
    catch (\Exception $exception) {
      // Logger une erreur en cas de problème.
      $this->logger->get('media_default_image')->error($exception->getMessage());
    }

    return $elements;
  }

  /**
   * Get default image file.
   */
  protected function getDefaultFile(): FileInterface {
    $fid = $this->config('media_default_image.settings')
      ->get('default_image');
    // If file is not exist get from entity.
    if (empty($fid)) {
      $default_scheme = $this->config('system.file')->get('default_scheme');
      $entity = $this->entityTypeManager
        ->getStorage('file')
        ->loadByProperties(['uri' => $default_scheme . '://media_default_image/image_default.jpg']);
      $file = reset($entity);
    }
    else {
      $file = $this->entityTypeManager->getStorage('file')->load($fid[0]);
    }

    return $file;
  }

  /**
   * Get Configuration Factory.
   *
   * @param string $variable
   *   The configuration data.
   *
   * @return \Drupal\Core\Config\ImmutableConfig
   *   System file configuration.
   */
  private function config(string $variable) {
    return $this->configFactory->get($variable);
  }

}
